/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.datastructures;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import java.awt.image.BufferedImage;
import java.awt.image.Raster;

import jspritemaker.toolkit.SpriteToolkit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class PixelGrid implements SpriteToolkit {
	private BufferedImage image;
	
	/* constructors */
	
	public PixelGrid(int width, int height) {
		this.initGrid(width,height);
	}
	
	/* private methods */
	
	private void initGrid(int width, int height) {
		this.image = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
		clearImage();
	}
	
	/* public methods */
	
	public BufferedImage getPixmap() {
		return this.image;
	}
	
	public int getWidth() {
		return this.image.getWidth();
	}
	
	public int getHeight() {
		return this.image.getHeight();
	}
	
	public void clearImage() {
		Graphics2D gfx = this.image.createGraphics();
		int width = image.getWidth(), height = image.getHeight();
		
		// erase the pixel map with white background color
		gfx.setColor(Color.white);
		gfx.fillRect(0,0,width,height);
	}
	
	public void resize(int width, int height) {
		int clipWidth = Math.min(width,getWidth());
		int clipHeight = Math.min(height,getHeight());
		Raster r;
		BufferedImage resized;
		Graphics2D gfx;
		
		resized = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
		gfx = resized.createGraphics();
		
		// clear the pixmap
		gfx.setColor(Color.white);
		gfx.fillRect(0,0,width,height);
		
		r = this.image.getData(new Rectangle(0,0,clipWidth,clipHeight));
		resized.setData(r);
		
		this.image = resized;
	}
	
	/* interfaces */
	
	// sprite toolkit interface
	public void drawPoint(Color c, int x, int y) {
		this.image.setRGB(x,y,c.getRGB());
	}
	
	public void drawLine(Color c, int x1, int y1, int x2, int y2) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.drawLine(x1,y1,x2,y2);
	}
	
	public void drawRectangle(Color c, int x, int y, int width, int height) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.drawRect(x,y,width,height);
	}
	
	public void fillRectangle(Color c, int x, int y, int width, int height) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.fillRect(x,y,width,height);
	}
	
	public void drawOval(Color c, int x, int y, int width, int height) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.drawOval(x,y,width,height);
	}
	
	public void fillOval(Color c, int x, int y, int width, int height) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.fillOval(x,y,width,height);
	}
	
	public void drawText(Color c, String text, int x, int y) {
		Graphics2D gfx = this.image.createGraphics();
		
		gfx.setColor(c);
		gfx.drawString(text,x,y);
	}
	
	public void fill(Color c, int x, int y) {
		// TODO: implement fill algorithm
	}
	
	// pixelmaps are just for modifying pixel data. selections should be handled
	// by the encapsulating class
	public void selectRectangle(int x, int y, int width, int height) {
		throw new UnsupportedOperationException("PixelMap should not select. Encapsulating class should handle selection!");
	}
}
