/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.Font;
import java.awt.Toolkit;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;

import java.awt.image.BufferedImage;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import jspritemaker.controller.CodePaneInput;

import jspritemaker.model.Sprite;
import jspritemaker.model.SpriteEdit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class CodeView extends JScrollPane {
	private Sprite model;
	private JTextArea text;
	
	private int width, height, count;
	
	// class constants
	public final static int LIGHT_PLANE     = 0;
	public final static int DARK_PLANE      = 1;
	public final static int MASK_PLANE      = 2;
	
	public final static int RGB_WHITE       = -1;
	public final static int RGB_BLACK       = -16777216;
	public final static int RGB_LIGHTGRAY   = -4934476;
	public final static int RGB_DARKGRAY    = -10855846;
	public final static int RGB_TRANSPARENT = -16760254;
	
	public final static String HEX_BEGIN    = "0x";
	public final static String HEX_NOT      = "~0x";
	public final static String HEX_COMMA    = ",";
	public final static String HEX_NEWLINE  = "\n";
	public final static String HEX_TAB      = "\t";
	public final static String HEX_UNSIGNED = "unsigned char";
	public final static String HEX_SHORT    = "unsigned short int";
	public final static String HEX_LONG     = "unsigned long int";
	public final static String HEX_SPACE    = " ";
	public final static String HEX_OPEN     = "[";
	public final static String HEX_CLOSE    = "] = {";
	public final static String HEX_END      = "};";
	
	/* constructors */
	
	protected CodeView(Sprite model) {
		super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		this.model = model;
		
		// initialize the scroll pane
		this.initPane();
	}
	
	/* private methods */
	
	private void initPane() {
		Font f = new Font("Monospaced",Font.PLAIN,12);
		
		this.text = new JTextArea(7,50);
		this.text.setTabSize(8);
		this.text.setFont(f);
		this.text.setEditable(false);
		this.update();
		
		this.setViewportView(text);
	}
	
	private String getDataType() {
		String datatype = HEX_UNSIGNED;
		
		if (this.width == 32) {
			datatype = HEX_LONG;
		} else if (this.width == 16) {
			datatype = HEX_SHORT;
		}
		
		return datatype;
	}
	
	private int getMiniSegments() {
		int segments = 1;
		
		if ((this.width == 24) || (this.width > 32)) {
			segments = width / 8;
		}

		return segments;
	}
	
	private String getDeclaration(String plane) {
		return new String(this.getDataType() + HEX_SPACE + plane + HEX_OPEN + (this.height * this.getMiniSegments()) + HEX_CLOSE + HEX_NEWLINE + HEX_TAB);
	}
	
	private int getHexWidth() {
		int hexWidth = 2;
		
		if (this.width == 32) {
			hexWidth = 8;
		} else if (this.width == 16) {
			hexWidth = 4;
		}
		
		return hexWidth;
	}
	
	private int convertRGB(int rgb, int plane) {
		int cRGB = 0;
		
		if (((rgb == RGB_LIGHTGRAY) || (rgb == RGB_BLACK)) && (plane == LIGHT_PLANE)) {
			cRGB = 1;
		} else if (((rgb == RGB_DARKGRAY) || (rgb == RGB_BLACK)) && (plane == DARK_PLANE)) {
			cRGB = 1;
		} else if ((rgb == RGB_TRANSPARENT) && (plane == MASK_PLANE)) {
			cRGB = 1;
		}
		
		return cRGB;
	}
	
	private String getHexDigit(int pixel, int plane) {
		BufferedImage pixmap = model.getPixmap();
		int loop, cPixel, digit = 0, x, y, segments = getMiniSegments();
		String hex;
		String[] digits = {"A","B","C","D","E","F"};
		
		for (loop = 0; loop < 4; loop++) {
			x = (pixel + loop) % width;
			y = (pixel + loop) / width;
			
			cPixel = this.convertRGB(pixmap.getRGB(x,y),plane);
			digit |= (cPixel << (3 - loop));
		}
		
		if (digit < 10) {
			hex = "" + digit;
		} else {
			digit -= 10;
			hex = digits[digit];
		}
		
		return hex;
	}
	
	private String getSegment(int number, int plane) {
		String segment, hexDigit;
		int hexWidth = getHexWidth();
		int segments = getMiniSegments();
		int currentPixel = (number * this.width), loop, loop2;
		
		if (hexWidth == 2) {
			segment = HEX_NOT;
		} else {
			segment = HEX_BEGIN;
		}
		
		for (loop = 0; loop < segments; loop++) {
			// for each mini-segment, create a hex marker
			
			for (loop2 = 0; loop2 < hexWidth; loop2++) {
				hexDigit = this.getHexDigit(currentPixel,plane);
				segment += hexDigit;
				currentPixel += 4;
			}
			
			// separate each mini-segment
			if ((loop + 1) < segments) {
				if (plane == MASK_PLANE) {
					segment += HEX_COMMA + HEX_NOT;
				} else {
					segment += HEX_COMMA + HEX_BEGIN;
				}
			}
		}

		return segment;
	}
	
	/* protected methods */
	
	protected void update() {
		SpriteEdit edit = model.getLastEdit();
		boolean doUpdate = false;
		
		if (edit == null) {
			doUpdate = true;
		} else {
			if (edit.isSignificant()) {
				doUpdate = true;
			}
		}
			
		if (doUpdate) {
			this.text.setText(getCode());
			this.text.setCaretPosition(0);
		}
	}
	
	protected void selectAll() {
		this.text.requestFocus();
		this.text.selectAll();
	}
	
	protected void copy() {
		Toolkit kit = Toolkit.getDefaultToolkit();
		Clipboard clipboard = kit.getSystemClipboard();
		StringSelection data;
		
		data = new StringSelection(text.getSelectedText());
		clipboard.setContents(data,data);
	}
	
	protected void setCodeInput(CodePaneInput input) {
		this.text.addMouseListener(input);
	}
	
	/* public methods */
	
	public String getCode() {
		BufferedImage sprite = model.getPixmap();
		String code;
		String light, dark, mask;
		
		int loop, sCount = 0, segments = getMiniSegments();
		
		// set the width and height
		this.width = sprite.getWidth();
		this.height = sprite.getHeight();
		
		light = getDeclaration(model.getLightPlaneName());
		dark = getDeclaration(model.getDarkPlaneName());
		mask = getDeclaration(model.getMaskName());
		
		// default count is 8 per row
		this.count = 8;
		
		for (loop = 0; loop < height; loop++) {
			light += getSegment(loop,LIGHT_PLANE);
			dark += getSegment(loop,DARK_PLANE);
			mask += getSegment(loop,MASK_PLANE);
			
			if (loop == (height - 1)) {
				light += HEX_NEWLINE + HEX_END;
				dark += HEX_NEWLINE + HEX_END;
				mask += HEX_NEWLINE + HEX_END;
			} else {
				light += HEX_COMMA;
				dark += HEX_COMMA;
				mask += HEX_COMMA;
			}
			
			sCount += segments;
			
			if (((sCount % count) == 0) && ((loop + 1) != height)) {
				light += HEX_NEWLINE + HEX_TAB;
				dark += HEX_NEWLINE + HEX_TAB;
				mask += HEX_NEWLINE + HEX_TAB;
			}
		}
		
		return (light + HEX_NEWLINE + dark + HEX_NEWLINE + mask);
	}
}
